Module: ConvenientService::Specification

Defined in:
lib/convenient_service/specification.rb

Constant Summary collapse

NAME =

Returns:

"convenient_service"
AUTHORS =

Returns:

["Marian Kostyk"].freeze
EMAIL =

Returns:

["[email protected]"].freeze
HOMEPAGE =

Returns:

"https://github.com/marian13/convenient_service"
SUMMARY =

Returns:

"Service object pattern implementation in Ruby.\n"
DESCRIPTION =

Returns:

"Yet another approach to revisit the service object pattern in Ruby, but this time focusing on the unique, opinionated, moderately obtrusive, but not mandatory features.\n"